<?php
/**
 * Plugin Name: Product Tag Cloud
 * Description: Display a list of related products based on common words in their names on single product pages.
 * Version: 1.12
 * Author: EasyToolsWP.com
 * Text Domain: product-tag-cloud
 */

// Register the settings and admin menu
add_action('admin_menu', 'ptc_add_admin_menu');
add_action('admin_init', 'ptc_settings_init');

function ptc_add_admin_menu() {
    add_submenu_page(
        'tools.php',
        __('Product Tag Cloud Settings', 'product-tag-cloud'),
        __('Product Tag Cloud', 'product-tag-cloud'),
        'manage_options',
        'product_tag_cloud',
        'ptc_options_page'
    );
}

function ptc_settings_init() {
    register_setting('ptcSettings', 'ptc_settings', [
        'sanitize_callback' => 'ptc_sanitize_settings'
    ]);

    add_settings_section(
        'ptc_section',
        __('Product Tag Cloud Settings', 'product-tag-cloud'),
        null,
        'ptcSettings'
    );

    add_settings_field(
        'ptc_word_count',
        __('Sensitivity', 'product-tag-cloud'),
        'ptc_word_count_render',
        'ptcSettings',
        'ptc_section'
    );

    add_settings_field(
        'ptc_filter_type',
        __('Filter By', 'product-tag-cloud'),
        'ptc_filter_type_render',
        'ptcSettings',
        'ptc_section'
    );

    add_settings_field(
        'ptc_exclude_words',
        __('Exclude Words', 'product-tag-cloud'),
        'ptc_exclude_words_render',
        'ptcSettings',
        'ptc_section'
    );

    add_settings_field(
        'ptc_bg_color',
        __('Tag Background Color', 'product-tag-cloud'),
        'ptc_bg_color_render',
        'ptcSettings',
        'ptc_section'
    );

    add_settings_field(
        'ptc_link_color',
        __('Tag Link Color', 'product-tag-cloud'),
        'ptc_link_color_render',
        'ptcSettings',
        'ptc_section'
    );

    add_settings_field(
        'ptc_no_related_text',
        __('No Related Products Text', 'product-tag-cloud'),
        'ptc_no_related_text_render',
        'ptcSettings',
        'ptc_section'
    );

    add_settings_field(
        'ptc_related_products_text',
        __('Related Products Text', 'product-tag-cloud'),
        'ptc_related_products_text_render',
        'ptcSettings',
        'ptc_section'
    );
}

function ptc_sanitize_settings($input) {
    $input['ptc_word_count'] = absint($input['ptc_word_count']);
    $input['ptc_filter_type'] = sanitize_text_field($input['ptc_filter_type']);
    $input['ptc_exclude_words'] = sanitize_text_field($input['ptc_exclude_words']);
    $input['ptc_bg_color'] = sanitize_hex_color($input['ptc_bg_color']);
    $input['ptc_link_color'] = sanitize_hex_color($input['ptc_link_color']);
    $input['ptc_no_related_text'] = sanitize_text_field($input['ptc_no_related_text']);
    $input['ptc_related_products_text'] = sanitize_text_field($input['ptc_related_products_text']);
    return $input;
}

function ptc_word_count_render() {
    $options = get_option('ptc_settings');
    $selected_value = $options['ptc_word_count'] ?? '2';
    ?>
    <select name='ptc_settings[ptc_word_count]'>
        <option value='2' <?php selected($selected_value, '2'); ?>><?php esc_html_e('Sensitive (2 words)', 'product-tag-cloud'); ?></option>
        <option value='3' <?php selected($selected_value, '3'); ?>><?php esc_html_e('Light (3 words)', 'product-tag-cloud'); ?></option>
        <option value='4' <?php selected($selected_value, '4'); ?>><?php esc_html_e('Very Relevant (4 words)', 'product-tag-cloud'); ?></option>
        <option value='5' <?php selected($selected_value, '5'); ?>><?php esc_html_e('Extra Relevant (5 words)', 'product-tag-cloud'); ?></option>
    </select>
    <?php
}

function ptc_filter_type_render() {
    $options = get_option('ptc_settings');
    ?>
    <select name='ptc_settings[ptc_filter_type]'>
        <option value='all' <?php selected($options['ptc_filter_type'] ?? 'all', 'all'); ?>><?php esc_html_e('All Products', 'product-tag-cloud'); ?></option>
        <option value='category' <?php selected($options['ptc_filter_type'] ?? 'all', 'category'); ?>><?php esc_html_e('Category', 'product-tag-cloud'); ?></option>
        <option value='tags' <?php selected($options['ptc_filter_type'] ?? 'all', 'tags'); ?>><?php esc_html_e('Tags', 'product-tag-cloud'); ?></option>
        <option value='category_tags' <?php selected($options['ptc_filter_type'] ?? 'all', 'category_tags'); ?>><?php esc_html_e('Category and Tags', 'product-tag-cloud'); ?></option>
    </select>
    <?php
}

function ptc_exclude_words_render() {
    $options = get_option('ptc_settings');
    $exclude_words = $options['ptc_exclude_words'] ?? 'for,as,it,your,our';
    ?>
    <input type='text' name='ptc_settings[ptc_exclude_words]' value='<?php echo esc_attr($exclude_words); ?>' />
    <p class="description"><?php esc_html_e('Comma-separated list of words to exclude from matching.', 'product-tag-cloud'); ?></p>
    <?php
}

function ptc_bg_color_render() {
    $options = get_option('ptc_settings');
    $bg_color = $options['ptc_bg_color'] ?? '#e0e0e0';
    ?>
    <input type='text' name='ptc_settings[ptc_bg_color]' value='<?php echo esc_attr($bg_color); ?>' class='color-picker' />
    <?php
}

function ptc_link_color_render() {
    $options = get_option('ptc_settings');
    $link_color = $options['ptc_link_color'] ?? '#0000ff';
    ?>
    <input type='text' name='ptc_settings[ptc_link_color]' value='<?php echo esc_attr($link_color); ?>' class='color-picker' />
    <?php
}

function ptc_no_related_text_render() {
    $options = get_option('ptc_settings');
    $no_related_text = $options['ptc_no_related_text'] ?? __('No related products found with the specified criteria.', 'product-tag-cloud');
    ?>
    <input type='text' name='ptc_settings[ptc_no_related_text]' value='<?php echo esc_attr($no_related_text); ?>' />
    <?php
}

function ptc_related_products_text_render() {
    $options = get_option('ptc_settings');
    $related_products_text = $options['ptc_related_products_text'] ?? __('Related Products', 'product-tag-cloud');
    ?>
    <input type='text' name='ptc_settings[ptc_related_products_text]' value='<?php echo esc_attr($related_products_text); ?>' />
    <?php
}

function ptc_options_page() {
    ?>
    <form action='options.php' method='post'>
        <h2><?php esc_html_e('Product Tag Cloud', 'product-tag-cloud'); ?></h2>
        <?php
        settings_fields('ptcSettings');
        do_settings_sections('ptcSettings');
        submit_button();
        ?>
    </form>
    <?php
}

// Add the related products list to single product pages
add_action('woocommerce_single_product_summary', 'ptc_display_related_products', 25);

function ptc_display_related_products() {
    global $post;

    $options = get_option('ptc_settings');
    $min_word_count = intval($options['ptc_word_count'] ?? 2);
    $filter_type = $options['ptc_filter_type'] ?? 'all';
    $exclude_words = array_map('trim', explode(',', $options['ptc_exclude_words'] ?? 'for,as,it,your,our'));
    $exclude_words = array_map('strtolower', $exclude_words);

    $args = array(
        'post_type' => 'product',
        'posts_per_page' => -1,
        'post_status' => 'publish',
        'post__not_in' => array($post->ID),
    );

    if ($filter_type === 'category') {
        $categories = wp_get_post_terms($post->ID, 'product_cat', array('fields' => 'ids'));
        $args['tax_query'] = array(
            array(
                'taxonomy' => 'product_cat',
                'field' => 'id',
                'terms' => $categories,
            )
        );
    } elseif ($filter_type === 'tags') {
        $tags = wp_get_post_terms($post->ID, 'product_tag', array('fields' => 'ids'));
        $args['tax_query'] = array(
            array(
                'taxonomy' => 'product_tag',
                'field' => 'id',
                'terms' => $tags,
            )
        );
    } elseif ($filter_type === 'category_tags') {
        $categories = wp_get_post_terms($post->ID, 'product_cat', array('fields' => 'ids'));
        $tags = wp_get_post_terms($post->ID, 'product_tag', array('fields' => 'ids'));
        $args['tax_query'] = array(
            'relation' => 'OR',
            array(
                'taxonomy' => 'product_cat',
                'field' => 'id',
                'terms' => $categories,
            ),
            array(
                'taxonomy' => 'product_tag',
                'field' => 'id',
                'terms' => $tags,
            ),
        );
    }

    $products = get_posts($args);

    if (empty($products)) {
        $no_related_text = $options['ptc_no_related_text'] ?? __('No related products found with the specified criteria.', 'product-tag-cloud');
        echo '<p>' . esc_html($no_related_text) . '</p>';
        return;
    }

    $current_product_words = str_word_count(strip_tags($post->post_title), 1);
    $words = array();

    foreach ($products as $product) {
        $product_name = $product->post_title;
        $product_words = str_word_count(strip_tags($product_name), 1);

        $current_product_words = array_diff(array_map('strtolower', $current_product_words), $exclude_words);
        $product_words = array_diff(array_map('strtolower', $product_words), $exclude_words);

        $common_words = array_intersect($current_product_words, $product_words);

        if (count($common_words) >= $min_word_count) {
            $words[$product->ID] = $product->post_title;
        }
    }

    if (empty($words)) {
        $no_related_text = $options['ptc_no_related_text'] ?? __('No related products found with the specified criteria.', 'product-tag-cloud');
        echo '<p>' . esc_html($no_related_text) . '</p>';
        return;
    }

    $bg_color = get_option('ptc_settings')['ptc_bg_color'] ?? '#e0e0e0';
    $link_color = get_option('ptc_settings')['ptc_link_color'] ?? '#0000ff';
    $related_products_text = $options['ptc_related_products_text'] ?? __('Related Products', 'product-tag-cloud');
    ?>
    <div class="ptc-related-products">
        <h3><?php echo esc_html($related_products_text); ?></h3>
        <ul>
            <?php foreach ($words as $product_id => $product_title) : ?>
                <li>
                    <a href="<?php echo esc_url(get_permalink($product_id)); ?>" style="background-color: <?php echo esc_attr($bg_color); ?>; color: <?php echo esc_attr($link_color); ?>;">
                        <?php echo esc_html($product_title); ?>
                    </a>
                </li>
            <?php endforeach; ?>
        </ul>
    </div>
    <style>
        .ptc-related-products ul {
            list-style: none;
            padding: 0;
        }

        .ptc-related-products li {
            margin: 0 0 10px;
        }

        .ptc-related-products a {
            text-decoration: none;
            display: block;
            padding: 10px;
        }

        .ptc-related-products a:hover {
            text-decoration: underline;
        }
    </style>
    <?php
}

// Enqueue Color Picker scripts
add_action('admin_enqueue_scripts', 'ptc_admin_enqueue_scripts');

function ptc_admin_enqueue_scripts($hook) {
    if ($hook !== 'tools_page_product_tag_cloud') {
        return;
    }
    wp_enqueue_style('wp-color-picker');
    wp_enqueue_script('ptc-color-picker', plugin_dir_url(__FILE__) . 'color-picker.js', array('wp-color-picker'), false, true);
}

// Load text domain for translations
add_action('plugins_loaded', 'ptc_load_textdomain');

function ptc_load_textdomain() {
    load_plugin_textdomain('product-tag-cloud', false, dirname(plugin_basename(__FILE__)) . '/languages');
}
